/*
    K8400 firmware modification based on original Version provided by Vellemann.

 Copyright (C) 2017 Holger Höhnemann

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef CARDREADER_H
#define CARDREADER_H

#include "MarlinHOH.h"

#ifdef SDSUPPORT

#include "SdFat.h"

#define MAX_DIR_DEPTH 10
#define MAX_VFAT_ENTRIES (2)
#define LONG_FILENAME_LENGTH (13*MAX_VFAT_ENTRIES+1)

enum CardStatus {No_Card = 0, Card_OK, Card_printing,Card_paused};

enum LsAction {LS_SerialPrint,LS_Count,LS_GetFilename};

class CardReader
{
public:
  CardReader();
  
  void initsd();
  void write_command(char *buf);
  //files auto[0-9].g on the sd card are performed in a row
  //this is to delay autostart and hence the initialisaiton of the sd card to some seconds after the normal init, so the device is available quick after a reset

  void checkautostart(bool x); 
  void openFile(char* name,bool read,bool replace_current=true);
  void openLogFile(char* name);
  void removeFile(char* name);
  void closefile(bool store_location=false);
  void release();
  void startFileprint();
  void pauseSDPrint();
  void getStatus();
  void printingHasFinished();

  void getfilename(const uint8_t nr);
  uint16_t getnrfilenames();
  
  void getAbsFilename(char *t, uint8_t nc);
  

  void ls();
  void chdir(const char * relpath);
  void updir();
  void setroot();
  bool IsDir();


  FORCE_INLINE bool isFileOpen() { return file.isOpen(); }
  FORCE_INLINE bool eof() { return file.available()==0 ;};
  FORCE_INLINE int16_t get() {  sdpos = file.curPosition();return (int16_t)file.read();};
  FORCE_INLINE void setIndex(long index) {sdpos = index;file.seekSet(index);};
  FORCE_INLINE uint8_t percentDone(){if(!isFileOpen()) return 0; if(filesize) return sdpos/((filesize+99)/100); else return 0;};
  FORCE_INLINE char* getWorkDirName(){sd.vwd()->getName(&my_printer.buffer[0],70); return my_printer.buffer;};
  FORCE_INLINE char* getPath(){return current_path;};
  char* FileName();


  bool saving;
  bool logging;
  enum CardStatus Status;
//  bool sdprinting ;
//  bool cardOK ;
//  char filename[13];
 // char longFilename[LONG_FILENAME_LENGTH];
  //bool filenameIsDir;
  int lastnr; //last number of the autostart;


private:
  FatFile root,*curDir,workDir,workDirParents[MAX_DIR_DEPTH];
  uint16_t workDirDepth;
  char current_path[100];
  char current_file[60];
//  Sd2Card card;
  SdFat sd;
  FatVolume volume;
  FatFile file;
  #define SD_PROCEDURE_DEPTH 1
  #define MAXPATHNAMELENGTH (1*MAX_DIR_DEPTH+MAX_DIR_DEPTH+1)
  uint8_t file_subcall_ctr;
  uint32_t filespos[SD_PROCEDURE_DEPTH];
  char filenames[SD_PROCEDURE_DEPTH][MAXPATHNAMELENGTH];
  uint32_t filesize;
  //int16_t n;
  unsigned long autostart_atmillis;
  uint32_t sdpos ;

  bool autostart_stilltocheck; //the sd start is delayed, because otherwise the serial cannot answer fast enought to make contact with the hostsoftware.
  
  LsAction lsAction; //stored for recursion.
  int16_t nrFiles; //counter for the files in the current directory and recycled as position counter for getting the nrFiles'th name in the directory.
  char* diveDirName;
  void LsDive(FatFile *parent, uint8_t flags, uint8_t indent);
};

extern CardReader sdcard;

#define IS_SD_PRINTING (sdcard.Status == Card_printing)

#if (SDCARDDETECT > -1)
# ifdef SDCARDDETECTINVERTED 
#  define IS_SD_INSERTED (READ(SDCARDDETECT)!=0)
# else
#  define IS_SD_INSERTED (READ(SDCARDDETECT)==0)
# endif //SDCARDTETECTINVERTED
#else
//If we don't have a card detect line, aways asume the card is inserted
# define IS_SD_INSERTED true
#endif

#else

#define IS_SD_PRINTING (false)

#endif //SDSUPPORT
#endif

