/*
    K8400 firmware modification based on original Version provided by Vellemann.

 Copyright (C) 2017 Holger Höhnemann

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef LANGUAGE_H
#define LANGUAGE_H

#include "Configuration.h"

#define GENERATE_LANGUAGE_INCLUDE(M)  STRINGIFY_(languages/language_##M.h)

// Macros to make a string from a macro
#define STRINGIFY_(M) #M
#define STRINGIFY(M) STRINGIFY_(M)

extern PGM_P message_list_en[];

#define TMESSAGE(A,B,C) const char Msg_##A##B[] PROGMEM =  C ;
#define TMESSAGEVAR(A,B) &Msg_##A##B[0]
#define T(a) my_printer.message_list_int[a]

enum Message_Strings  {
		MSG_Marlin = 0,
		//---------------- LCD messages: status messages
		MSG_Wait,
		MSG_Welcome,
		MSG_Heating,
		MSG_Heating_complete,
		MSG_Printing,
		//---------------- LCD messages: Menus messages
		MSG_Main_Menu,
		MSG_Info,
		MSG_Tune,
		MSG_Prepare,
		MSG_Control,
		MSG_Firmware,
		MSG_Auto_Home,
		MSG_Load_Filament,
		MSG_Unload_Filament,
		MSG_Preheat_PLA,
		MSG_Preheat_ABS,
		MSG_Move_Axis,
		MSG_Disable_Steppers,
		MSG_Cooldown,
		MSG_Extruder_offsets,
		MSG_Temperature,
		MSG_Motion,
		MSG_Advanced,
		MSG_Stopwatch,
		MSG_Store_Eprom,
		MSG_Load_Eprom,
		MSG_Restore_Failsave,
		MSG_Load_ABS_EX1,
		MSG_Unload_ABS_EX1,
		MSG_Load_ABS_EX2,
		MSG_Unload_ABS_EX2,
		MSG_Load_PLA_EX1,
		MSG_Unload_PLA_EX1,
		MSG_Load_PLA_EX2,
		MSG_Unload_PLA_EX2,
		MSG_Preheat_PLA_EX1,
		MSG_Preheat_PLA_EX2,
		MSG_Preheat_PLA_EXALL,
		MSG_Preheat_ABS_EX1,
		MSG_Preheat_ABS_EX2,
		MSG_Preheat_ABS_EXALL,
		MSG_Load_TEXT0,
		MSG_Load_TEXT1,
		MSG_Load_TEXT2,
		MSG_Load_TEXT3,
		MSG_No_Card,
		MSG_Pause_Print,
		MSG_Resume_Print,
		MSG_Stop_Print,
		MSG_Card_Menu,
		MSG_Init_SDcard,
		MSG_Change_SDcard,
		MSG_Refresh,
		MSG_Folder_Up,
		MSG_Speed,
		MSG_Move_01mm,
		MSG_Move_1mm,
		MSG_Move_10mm,
		MSG_Move_X,
		MSG_Move_Y,
		MSG_Move_Z,
		MSG_Move_E,
		MSG_Start_Timer,
		MSG_Stop_Timer,
		MSG_Error_Maxtemp,
		MSG_Error_Mintemp,
		//-----------------LCD-format-Strings
		MSG_Nozzle,
		MSG_Nozzle1,
		MSG_Nozzle2,
		MSG_Bed,
		MSG_FanSpeed,
		MSG_Adv_Press1,
		MSG_Adv_Press2,
		MSG_Adv_delay1,
		MSG_Adv_delay2,
		MSG_Flow,
		MSG_Flow1,
		MSG_Flow2,
		MSG_PID_P0,
		MSG_PID_I0,
		MSG_PID_D0,
		MSG_PID_C0,
		MSG_PID_F0,
		MSG_PID_P1,
		MSG_PID_I1,
		MSG_PID_D1,
		MSG_PID_C1,
		MSG_PID_F1,
		MSG_PID_P99,
		MSG_PID_I99,
		MSG_PID_D99,
		MSG_PID_C99,
		MSG_PID_F99,
		MSG_Accel,
		MSG_VXY_Jerk,
		MSG_VZ_Jerk,
		MSG_VE_Jerk,
		MSG_Vmax_X,
		MSG_Vmax_Y,
		MSG_Vmax_Z,
		MSG_Vmax_E,
		MSG_Vmin,
		MSG_Vtrav_min,
		MSG_A_Retract,
		MSG_A_Travel,
		MSG_Xsteps,
		MSG_Ysteps,
		MSG_Zsteps,
		MSG_Esteps,
		MSG_Flow_time,
		MSG_Amax_X,
		MSG_Amax_Y,
		MSG_Amax_Z,
		MSG_Amax_E,
		MSG_Extruder_offset_X,
		MSG_Extruder_offset_Y,
		//-----------------Serial port messages
		MSG_SD_Init_Fail,
		MSG_SD_Vol_Init_Fail,
		MSG_SD_Vol_Type,
		MSG_SD_Card_OK,
		MSG_Begin_File_List,
		MSG_End_File_List,
		MSG_SD_File_Opened,
		MSG_SD_Open_File_Fail,
		MSG_SD_Printing_Byte,
		MSG_SD_Not_Printing,
		MSG_Endstops_Hit,
		MSG_CS_Error_Load_Failsave,
		MSG_CS_Value,
		MSG_Invalid_Extruder,
		MSG_Invalid_Extruder_Num,
		MSG_Active_Extruder,
		MSG_PID_Autotune_Invalid_Extrude,
		MSG_PID_Autotune_Start,
		MSG_Hotend_Stabilise,
		MSG_Hotend_Stable_Temperature,
		MSG_PID_Autotune_Failed,
		MSG_PID_Autotune_Timeout,
		MSG_PID_Autotune_Finished,
		MSG_PID_Autotune_Finished_Updated,
		MSG_Temp_Tune_Failed,
		MSG_Temp_Tune_Started,
		MSG_Temp_Tune_Fan,
		MSG_Temp_Tune_Extrusion,
		MSG_Temp_Heat_Result,
		MSG_Temp_Heat_KC,
		MSG_Maxtemp_Error_Extr,
		MSG_Mintemp_Error_Extr,
		MSG_Fan_Autotune_Started,
		MSG_Repeat_Temperature_Stabilisation,
		MSG_Temperature_Stabilisation_Abort_At,
		MSG_Cold_Extrusion_Prevented,
		MSG_Too_Long_Extrusion_Prevented,
		MSG_Default_Parameters_Loaded,
		MSG_NB_EEPROM_Bytes_Loaded,
		MSG_Steps_Per_Unit,
		MSG_NB_Steps_Per_Unit,
		MSG_Max_Feedrates,
		MSG_NB_Max_Feedrates,
		MSG_Max_Acceleration,
		MSG_NB_Max_Acceleration,
		MSG_Unknown_Command,
		MSG_Filamentchange,
		MSG_Extruder_Nb,
		MSG_PID_Parameter,
		MSG_CF_Parameter
};

TMESSAGE(en,MSG_Marlin,"Marlin_HOH ");
//---------------- LCD messages: status messages
TMESSAGE(en, MSG_Wait, "wait");
TMESSAGE(en, MSG_Welcome, "K8400 ready.");
TMESSAGE(en, MSG_Heating,  "Heating...");
TMESSAGE(en, MSG_Heating_complete,  "Heating done.");
TMESSAGE(en, MSG_Printing,  "Printing...");

//---------------- LCD messages: Menus messages
TMESSAGE(en, MSG_Main_Menu, "Main");
TMESSAGE(en, MSG_Info, "Info Screen");
TMESSAGE(en, MSG_Tune, "Tune");
TMESSAGE(en, MSG_Prepare, "Prepare");
TMESSAGE(en, MSG_Control, "Control");
TMESSAGE(en, MSG_Firmware ,"Firmware version");
TMESSAGE(en, MSG_Auto_Home , "Auto home");
TMESSAGE(en, MSG_Load_Filament, "Load filament");
TMESSAGE(en, MSG_Unload_Filament, "Unload filament");
TMESSAGE(en, MSG_Preheat_PLA, "Preheat PLA");
TMESSAGE(en, MSG_Preheat_ABS, "Preheat ABS");
TMESSAGE(en, MSG_Move_Axis, "Move axis");
TMESSAGE(en, MSG_Disable_Steppers, "Disable steppers");
TMESSAGE(en, MSG_Cooldown, "Cooldown");
TMESSAGE(en, MSG_Extruder_offsets, "Extruder offsets");
TMESSAGE(en, MSG_Temperature, "Temperature");
TMESSAGE(en, MSG_Motion, "Motion");
TMESSAGE(en, MSG_Advanced, "Advanced");
TMESSAGE(en, MSG_Stopwatch, "Stopwatch");
TMESSAGE(en, MSG_Store_Eprom, "Store memory");
TMESSAGE(en, MSG_Load_Eprom, "Load memory");
TMESSAGE(en, MSG_Restore_Failsave, "Restore failsafe");
TMESSAGE(en, MSG_Load_ABS_EX1, "Load ABS right");
TMESSAGE(en, MSG_Unload_ABS_EX1, "Unload ABS right");
TMESSAGE(en, MSG_Load_ABS_EX2, "Load ABS left");
TMESSAGE(en, MSG_Unload_ABS_EX2, "Unload ABS left");
TMESSAGE(en, MSG_Load_PLA_EX1, "Load PLA right");
TMESSAGE(en, MSG_Unload_PLA_EX1, "Unload PLA right");
TMESSAGE(en, MSG_Load_PLA_EX2, "Load PLA left");
TMESSAGE(en, MSG_Unload_PLA_EX2, "Unload PLA left");
TMESSAGE(en, MSG_Preheat_PLA_EX1, "Preheat PLA 1");
TMESSAGE(en, MSG_Preheat_PLA_EX2, "Preheat PLA 2");
TMESSAGE(en, MSG_Preheat_PLA_EXALL, "Preheat PLA All");
TMESSAGE(en, MSG_Preheat_ABS_EX1, "Preheat ABS 1");
TMESSAGE(en, MSG_Preheat_ABS_EX2, "Preheat ABS 2");
TMESSAGE(en, MSG_Preheat_ABS_EXALL, "Preheat ABS All");
TMESSAGE(en, MSG_Load_TEXT0, "Insert filament");
TMESSAGE(en, MSG_Load_TEXT1, " until 1.5cm is");
TMESSAGE(en, MSG_Load_TEXT2, " visible in tube");
TMESSAGE(en, MSG_Load_TEXT3, "Then press button");
TMESSAGE(en, MSG_No_Card , "No SD card");
TMESSAGE(en, MSG_Pause_Print , "Pause print");
TMESSAGE(en, MSG_Resume_Print, "Resume print");
TMESSAGE(en, MSG_Stop_Print,  "Stop print");
TMESSAGE(en, MSG_Card_Menu , "Print from SD");
TMESSAGE(en, MSG_Init_SDcard , "Init. SD card");
TMESSAGE(en, MSG_Change_SDcard , "Change SD card");
TMESSAGE(en, MSG_Refresh, "\x5Refresh");
TMESSAGE(en, MSG_Folder_Up, "\x05..");
TMESSAGE(en, MSG_Speed, "Speed");
TMESSAGE(en, MSG_Move_01mm,  "Move 0.1mm");
TMESSAGE(en, MSG_Move_1mm,   "Move 1mm");
TMESSAGE(en, MSG_Move_10mm,  "Move 10mm");
TMESSAGE(en, MSG_Move_X,     "Move X");
TMESSAGE(en, MSG_Move_Y ,    "Move Y");
TMESSAGE(en, MSG_Move_Z ,    "Move Z");
TMESSAGE(en, MSG_Move_E,     "Extruder");
TMESSAGE(en, MSG_Start_Timer,     "Start");
TMESSAGE(en, MSG_Stop_Timer,     "Stop");
TMESSAGE(en, MSG_Error_Maxtemp,     "Err: MAXTEMP");
TMESSAGE(en, MSG_Error_Mintemp,     "Err: MINTEMP");
//-----------------LCD-format-Strings
TMESSAGE(en, MSG_Nozzle, 	"Nozzle         %3i");
TMESSAGE(en, MSG_Nozzle1, 	"Nozzle 1       %3i");
TMESSAGE(en, MSG_Nozzle2, 	"Nozzle 2       %3i");
TMESSAGE(en, MSG_Bed,     	"Bed            %3i");
TMESSAGE(en, MSG_FanSpeed  ,"Fan speed      %3i");
TMESSAGE(en, MSG_Adv_Press1,"press_adv_1    %3.0f");
TMESSAGE(en, MSG_Adv_Press2,"press_adv_2    %3.0f");
TMESSAGE(en, MSG_Adv_delay1,"delay 1        %3i");
TMESSAGE(en, MSG_Adv_delay2,"delay 2        %3i");
TMESSAGE(en, MSG_Flow, 		"Flow           %3i");
TMESSAGE(en, MSG_Flow1,   	"Flow 1         %3i");
TMESSAGE(en, MSG_Flow2,     "Flow 2         %3i");
TMESSAGE(en, MSG_PID_P0,    "PID-P 1      %5.2f");
TMESSAGE(en, MSG_PID_I0,    "PID-I 1      %5.2f");
TMESSAGE(en, MSG_PID_D0,    "PID-D 1      %5.2f");
TMESSAGE(en, MSG_PID_C0,    "PID-C 1      %5.2f");
TMESSAGE(en, MSG_PID_F0,    "PID-F 1        %3.0f");
TMESSAGE(en, MSG_PID_P1,    "PID-P 2      %5.2f");
TMESSAGE(en, MSG_PID_I1,    "PID-I 2      %5.2f");
TMESSAGE(en, MSG_PID_D1,    "PID-D 2      %5.2f");
TMESSAGE(en, MSG_PID_C1,    "PID-C 2      %5.2f");
TMESSAGE(en, MSG_PID_F1,    "PID-F 2        %3.0f");
TMESSAGE(en, MSG_PID_P99,    "PID-P Bed    %5.2f");
TMESSAGE(en, MSG_PID_I99,    "PID-I Bed    %5.2f");
TMESSAGE(en, MSG_PID_D99,    "PID-D Bed    %5.2f");
TMESSAGE(en, MSG_PID_C99,    "PID-C Bed    %5.2f");
TMESSAGE(en, MSG_PID_F99,    "PID-F Bed      %3.0f");
TMESSAGE(en, MSG_Accel,		"Accel         %5.0f");
TMESSAGE(en, MSG_VXY_Jerk,  "Vxy-jerk       %3.0f");
TMESSAGE(en, MSG_VZ_Jerk,   "Vz-jerk       %5.2f");
TMESSAGE(en, MSG_VE_Jerk,   "Ve-jerk        %3.0f");
TMESSAGE(en, MSG_Vmax_X,    "Vmax X         %3.0f");
TMESSAGE(en, MSG_Vmax_Y,    "Vmax Y         %3.0f");
TMESSAGE(en, MSG_Vmax_Z,    "Vmax Z         %3.0f");
TMESSAGE(en, MSG_Vmax_E,    "Vmax E        %4.0f");
TMESSAGE(en, MSG_Vmin,      "Vmin           %3.0f");
TMESSAGE(en, MSG_Vtrav_min, "VTrav min      %3.0f");
TMESSAGE(en, MSG_A_Retract, "A-retract     %5.0f");
TMESSAGE(en, MSG_A_Travel,  "A-travel");
TMESSAGE(en, MSG_Xsteps,    "Xsteps/mm   %5.2f");
TMESSAGE(en, MSG_Ysteps,    "Ysteps/mm   %5.2f");
TMESSAGE(en, MSG_Zsteps,    "Zsteps/mm   %5.1f");
TMESSAGE(en, MSG_Esteps,    "Esteps/mm    %5.1f");
TMESSAGE(en, MSG_Flow_time, "Flow time      %3i");
TMESSAGE(en, MSG_Amax_X,    "Amax X    %5l");
TMESSAGE(en, MSG_Amax_Y,    "Amax Y    %5l");
TMESSAGE(en, MSG_Amax_Z,    "Amax Z    %5l");
TMESSAGE(en, MSG_Amax_E,    "Amax E    %5l");
TMESSAGE(en, MSG_Extruder_offset_X,    "Offset X    %1.2f");
TMESSAGE(en, MSG_Extruder_offset_Y,    "Offset Y    %1.2f");
//-----------------Serial port messages: first part must not be translated, they are necessary for communication with
//-----------------cpntroller software
TMESSAGE(en, MSG_Active_Extruder , "Active Extruder: E%i\n");
TMESSAGE(en, MSG_Unknown_Command , "unknown command: \\%s\\ \n");
TMESSAGE(en, MSG_Begin_File_List,  "Begin file list\n");
TMESSAGE(en, MSG_End_File_List,  "End file list\n");
TMESSAGE(en, MSG_Invalid_Extruder, "echo: M%i Invalid extruder T%i\n");
TMESSAGE(en, MSG_SD_Init_Fail, "SD init fail");
TMESSAGE(en, MSG_SD_Card_OK ,  "SD card ok\n");
TMESSAGE(en, MSG_SD_File_Opened, "File opened: %s Size: %i\n");
TMESSAGE(en, MSG_SD_Open_File_Fail, "open failed, File: %s\n");
TMESSAGE(en, MSG_SD_Printing_Byte, "SD printing byte %i / %i\n");
TMESSAGE(en, MSG_SD_Not_Printing ,  "Not SD printing\n");
TMESSAGE(en, MSG_Endstops_Hit ,  "endstops hit: ");
//---------------The following messages can be teanslated
TMESSAGE(en, MSG_CS_Error_Load_Failsave , "Checksum error on printer parameters. Failsave values loaded\n");
TMESSAGE(en, MSG_CS_Value , "Checksum :  %i\n");
TMESSAGE(en, MSG_SD_Vol_Init_Fail , "Can't find a valid FAT16/FAT32 partition.\n");
TMESSAGE(en, MSG_SD_Vol_Type , "Volume is FAT%i \n");
TMESSAGE(en, MSG_Invalid_Extruder_Num, "error: T%i Invalid extruder number\n");
TMESSAGE(en, MSG_Filamentchange, "Change filament\n" );
TMESSAGE(en, MSG_PID_Autotune_Invalid_Extrude , "PID Autotune failed. Bad extruder number.");
TMESSAGE(en, MSG_PID_Autotune_Start , "PID Autotune start");
TMESSAGE(en, MSG_Hotend_Stabilise , "Stabilise hotend\n");
TMESSAGE(en, MSG_Hotend_Stable_Temperature , "Temperature stabilised at %f\n");
TMESSAGE(en, MSG_PID_Autotune_Failed , "PID Autotune failed! Temperature too high");
TMESSAGE(en, MSG_PID_Autotune_Timeout , "PID Autotune failed! timeout\n");
TMESSAGE(en, MSG_PID_Autotune_Finished , "PID Autotune finished! Put the last Kp, Ki and Kd constants from above into Configuration.h\n");
TMESSAGE(en, MSG_PID_Autotune_Finished_Updated , "PID Autotune finished! PID parameters updated\n");
TMESSAGE(en, MSG_Temp_Tune_Failed , "Abort tuning, not at home position\n");
TMESSAGE(en, MSG_Temp_Tune_Started , "Start tuning - - -\n");
TMESSAGE(en, MSG_Temp_Tune_Fan , "Step 1: tune fan parameter\n");
TMESSAGE(en, MSG_Temp_Tune_Extrusion , "Step 2: tune extrusion parameter");
TMESSAGE(en, MSG_Temp_Heat_Result , " Heat @ %imm : %i\n");
TMESSAGE(en, MSG_Temp_Heat_KC , " Extrusion coefficient Kc %i\n");
TMESSAGE(en, MSG_Maxtemp_Error_Extr , " Heater switched off. MAXTEMP triggered on Extruder %i!\n");
TMESSAGE(en, MSG_Mintemp_Error_Extr , " Heater switched off. MINTEMP triggered on Extruder %i!\n");
TMESSAGE(en, MSG_Fan_Autotune_Started ,"Fan Autotune starting\n");
TMESSAGE(en, MSG_Repeat_Temperature_Stabilisation , " Repeat Temperature stabilisation \n");
TMESSAGE(en, MSG_Temperature_Stabilisation_Abort_At , " Temperature not reached, abort at %.1f\n");
TMESSAGE(en, MSG_Cold_Extrusion_Prevented , " cold extrusion prevented\n");
TMESSAGE(en, MSG_Too_Long_Extrusion_Prevented , " too long extrusion prevented\n");
TMESSAGE(en, MSG_Default_Parameters_Loaded , "Hardcoded Default Parameter Loaded");
TMESSAGE(en, MSG_NB_EEPROM_Bytes_Loaded , " eeprom bytes loaded: %i\n");
TMESSAGE(en, MSG_Steps_Per_Unit , "Steps per unit (M92):\n");
TMESSAGE(en, MSG_NB_Steps_Per_Unit , " X %1.2f, Y %1.2f, Z %1.2f, E %1.2f\n\n");
TMESSAGE(en, MSG_Max_Feedrates , "Maximum feedrates (mm/s) (M203)\n:");
TMESSAGE(en, MSG_NB_Max_Feedrates , "  M203 X %f, Y %f, Z %f, E %f\n\n");
TMESSAGE(en, MSG_Max_Acceleration , "Maximum Acceleration (mm/s2) (M201):\n");
TMESSAGE(en, MSG_NB_Max_Acceleration , "  M201 X %l, Y %l, Z %l, E %l\n\n");
TMESSAGE(en, MSG_Extruder_Nb , " Heater  %i:\n");
TMESSAGE(en, MSG_PID_Parameter , "  P: %f I: %f D: %f\n");
TMESSAGE(en, MSG_CF_Parameter , "  C: %f F: %f\n\n");
// NOTE: IF YOU CHANGE LANGUAGE FILES OR MERGE A FILE WITH CHANGES
//
//   ==> ALWAYS TRY TO COMPILE MARLIN WITH/WITHOUT "ULTIPANEL" / "ULTRALCD" / "SDSUPPORT" #define IN "Configuration.h"
//   ==> ALSO TRY ALL AVAILABLE LANGUAGE OPTIONS
// See also https://github.com/MarlinFirmware/Marlin/wiki/LCD-Language

// Languages
// en         English
// pl         Polish
// fr         French
// de         German
// es         Spanish
// ru         Russian
// bg         Bulgarian
// it         Italian
// pt         Portuguese
// pt_utf8    Portuguese (UTF8)
// pt-br      Portuguese (Brazilian)
// pt-br_utf8 Portuguese (Brazilian UTF8)
// fi         Finnish
// an         Aragonese
// nl         Dutch
// gl         Galician
// ca         Catalan
// eu         Basque-Euskera
// kana       Japanese
// kana_utf8  Japanese (UTF8)
// cn         Chinese
// cz         Czech

// fallback if no language is set, don't change
#ifndef LANGUAGE_INCLUDE
  #define LANGUAGE_INCLUDE GENERATE_LANGUAGE_INCLUDE(en)
#endif

#if defined USE_AUTOMATIC_VERSIONING
  #include "_Version.h"
#else
  #include "Version.h"
#endif

#ifdef DEFAULT_SOURCE_CODE_URL
  #undef  SOURCE_CODE_URL
  #define SOURCE_CODE_URL DEFAULT_SOURCE_CODE_URL
#endif

#ifdef CUSTOM_MACHINE_NAME
  #undef  MACHINE_NAME
  #define MACHINE_NAME CUSTOM_MACHINE_NAME
#else
  #ifdef DEFAULT_MACHINE_NAME
    #undef  MACHINE_NAME
    #define MACHINE_NAME DEFAULT_MACHINE_NAME
  #endif
#endif

#ifndef MACHINE_UUID
  #define MACHINE_UUID DEFAULT_MACHINE_UUID
#endif

#ifdef DEFAULT_WEBSITE_URL
  #undef  WEBSITE_URL
  #define WEBSITE_URL DEFAULT_WEBSITE_URL
#endif

// Common LCD messages
  #define MSG_M104_INVALID_EXTRUDER "M104 Invalid extruder "
  #define MSG_M105_INVALID_EXTRUDER "M105 Invalid extruder "
  #define MSG_M200_INVALID_EXTRUDER "M200 Invalid extruder "
  #define MSG_M218_INVALID_EXTRUDER "M218 Invalid extruder "
  #define MSG_M221_INVALID_EXTRUDER "M221 Invalid extruder "
 #define MSG_M109_INVALID_EXTRUDER "M109 Invalid extruder "
        #define MSG_EXTRUDER_1 "Right (EXT.1)"
        #define MSG_EXTRUDER_2 "Left (EXT.2)"

  #define MSG_FLOW0 "Flow 1"
  #define MSG_FLOW1 "Flow 2"
  #define MSG_FLOW2 "Flow 3"
        #define MSG_LED_MENU "Set LED color"
        #define MSG_LOAD_FILAMENT "Load filament"
        #define MSG_UNLOAD_FILAMENT "Unload filament"
        #define MSG_LOAD_ABS_EX1 "Load ABS right"
        #define MSG_UNLOAD_ABS_EX1 "Unload ABS right"
        #define MSG_LOAD_ABS_EX2 "Load ABS left"
        #define MSG_UNLOAD_ABS_EX2 "Unload ABS left"
        #define MSG_LOAD_PLA_EX1 "Load PLA right"
        #define MSG_UNLOAD_PLA_EX1 "Unload PLA right"
        #define MSG_LOAD_PLA_EX2 "Load PLA left"
        #define MSG_UNLOAD_PLA_EX2 "Unload PLA left"
        #define MSG_LOAD_TEXT0 " Insert filament"
        #define MSG_LOAD_TEXT1 " until 1.5cm is"
        #define MSG_LOAD_TEXT2 " visible in tube"
        #define MSG_LOAD_TEXT3 "Then press button"
  #define MSG_PREHEAT_PLA0 "Preheat PLA 1"
  #define MSG_PREHEAT_PLA1 "Preheat PLA 2"
  #define MSG_PREHEAT_PLA2 "Preheat PLA 3"
  #define MSG_PREHEAT_PLA012 "Preheat PLA All"
  #define MSG_PREHEAT_ABS0 "Preheat ABS 1"
  #define MSG_PREHEAT_ABS1 "Preheat ABS 2"
  #define MSG_PREHEAT_ABS2 "Preheat ABS 3"
  #define MSG_PREHEAT_ABS012 "Preheat ABS All"
  #define MSG_NOZZLE1 "Nozzle 2"
  #define MSG_NOZZLE2 "Nozzle 3"
        #define MSG_LED_MENU "Set LED color"
        #define MSG_LED_BLUE "Blue"
        #define MSG_LED_LIGHTBLUE "Lightblue"
        #define MSG_LED_PURPLE "Purple"
        #define MSG_LED_RED "Red"
        #define MSG_LED_GREEN "Green"
        #define MSG_LED_YELLOW "Yellow"
        #define MSG_LED_WHITE "White"
        #define MSG_LED_OFF "Off"

// LCD Splashscreen Messages
// Please note these can be 20 characters!

        #define MSG_SPLASH_NAME " VERTEX  3D PRINTER"
        #define MSG_SPLASH_FIRMWARE "M:1.0 V:1.4h4 HEADS:2"
        #define MSG_SPLASH_WEBSITE1 " vertex3dprinter.eu"
        #define MSG_SPLASH_WEBSITE2 "k8400.hoehnemann.net"

        #define MSG_SPLASH_NAME_SMALL "VERTEX 3D PRINTER"
        #define MSG_SPLASH_FIRMWARE_SMALL "M:1.0 V:1.4h4 HDS:2"
        #define MSG_SPLASH_WEBSITE1_SMALL "     K8400.eu"
        #define MSG_SPLASH_WEBSITE2_SMALL "k8400.hoehnemann.net"
  /* nothing here yet */

// Common serial messages
//#define MSG_MARLIN "Marlin_HOH "

// Serial Console Messages (do not translate those!)

#define MSG_Enqueueing                      "enqueueing \""
#define MSG_POWERUP                         "PowerUp"
#define MSG_EXTERNAL_RESET                  " External Reset"
#define MSG_BROWNOUT_RESET                  " Brown out Reset"
#define MSG_WATCHDOG_RESET                  " Watchdog Reset"
#define MSG_SOFTWARE_RESET                  " Software Reset"
#define MSG_AUTHOR                          " | Author: "
#define MSG_CONFIGURATION_VER               " Last Updated: "
#define MSG_FREE_MEMORY                     " Free Memory: "
#define MSG_PLANNER_BUFFER_BYTES            "  PlannerBufferBytes: "
#define MSG_OK                              "ok"
#define MSG_WAIT                            "wait"
#define MSG_FILE_SAVED                      "Done saving file."
#define MSG_ERR_LINE_NO                     "Line Number is not Last Line Number+1, Last Line: "
#define MSG_ERR_CHECKSUM_MISMATCH           "checksum mismatch, Last Line: "
#define MSG_ERR_NO_CHECKSUM                 "No Checksum with line number, Last Line: "
#define MSG_ERR_NO_LINENUMBER_WITH_CHECKSUM "No Line Number with checksum, Last Line: "
#define MSG_FILE_PRINTED                    "Done printing file"
#define MSG_BEGIN_FILE_LIST                 "Begin file list"
#define MSG_END_FILE_LIST                   "End file list"
#define MSG_INVALID_EXTRUDER                "Invalid extruder"
#define MSG_INVALID_SOLENOID                "Invalid solenoid"
#define MSG_ERR_NO_THERMISTORS              "No thermistors - no temperature"
#define MSG_M115_REPORT                     "FIRMWARE_NAME:Marlin " DETAILED_BUILD_VERSION " SOURCE_CODE_URL:" SOURCE_CODE_URL " PROTOCOL_VERSION:" PROTOCOL_VERSION " MACHINE_TYPE:" MACHINE_NAME " EXTRUDER_COUNT:" STRINGIFY(EXTRUDERS) " UUID:" MACHINE_UUID "\n"
#define MSG_COUNT_X                         " Count X: "
#define MSG_COUNT_A                         " Count A: "
#define MSG_ERR_KILLED                      "Printer halted. kill() called!"
#define MSG_ERR_STOPPED                     "Printer stopped due to errors. Fix the error and use M999 to restart. (Temperature is reset. Set it after restarting)"
#define MSG_BUSY_PROCESSING                 "busy: processing"
#define MSG_BUSY_PAUSED_FOR_USER            "busy: paused for user"
#define MSG_BUSY_PAUSED_FOR_INPUT           "busy: paused for input"
#define MSG_RESEND                          "Resend: "
#define MSG_UNKNOWN_COMMAND                 "Unknown command: \""
#define MSG_ACTIVE_EXTRUDER                 "Active Extruder: "
#define MSG_X_MIN                           "x_min: "
#define MSG_X_MAX                           "x_max: "
#define MSG_Y_MIN                           "y_min: "
#define MSG_Y_MAX                           "y_max: "
#define MSG_Z_MIN                           "z_min: "
#define MSG_Z_MAX                           "z_max: "
#define MSG_Z2_MAX                          "z2_max: "
#define MSG_Z_PROBE                         "z_probe: "
#define MSG_ERR_MATERIAL_INDEX              "M145 S<index> out of range (0-1)"
#define MSG_ERR_M421_REQUIRES_XYZ           "M421 requires XYZ parameters"
#define MSG_ERR_MESH_INDEX_OOB              "Mesh XY index is out of bounds"
#define MSG_ERR_M428_TOO_FAR                "Too far from reference point"
#define MSG_ERR_M303_DISABLED               "PIDTEMP disabled"
#define MSG_M119_REPORT                     "Reporting endstop status"
#define MSG_ENDSTOP_HIT                     "TRIGGERED"
#define MSG_ENDSTOP_OPEN                    "open"
#define MSG_HOTEND_OFFSET                   "Hotend offsets:"

#define MSG_SD_CANT_OPEN_SUBDIR             "Cannot open subdir"
#define MSG_SD_INIT_FAIL                    "SD init fail"
#define MSG_SD_VOL_INIT_FAIL                "volume.init failed"
#define MSG_SD_OPENROOT_FAIL                "openRoot failed"
#define MSG_SD_CARD_OK                      "SD card ok"
#define MSG_SD_WORKDIR_FAIL                 "workDir open failed"
#define MSG_SD_OPEN_FILE_FAIL               "open failed, File: "
#define MSG_SD_FILE_OPENED                  "File opened: "
#define MSG_SD_SIZE                         " Size: "
#define MSG_SD_FILE_SELECTED                "File selected"
#define MSG_SD_WRITE_TO_FILE                "Writing to file: "
#define MSG_SD_PRINTING_BYTE                "SD printing byte "
#define MSG_SD_NOT_PRINTING                 "Not SD printing"
#define MSG_SD_ERR_WRITE_TO_FILE            "error writing to file"
#define MSG_SD_CANT_ENTER_SUBDIR            "Cannot enter subdir: "

#define MSG_STEPPER_TOO_HIGH                "Steprate too high: "
#define MSG_ENDSTOPS_HIT                    "endstops hit: "
#define MSG_ERR_COLD_EXTRUDE_STOP           " cold extrusion prevented"
#define MSG_ERR_LONG_EXTRUDE_STOP           " too long extrusion prevented"
#define MSG_TOO_COLD_FOR_M600               "M600 Hotend too cold to change filament"
#define MSG_BABYSTEPPING_X                  "Babystepping X"
#define MSG_BABYSTEPPING_Y                  "Babystepping Y"
#define MSG_BABYSTEPPING_Z                  "Babystepping Z"
#define MSG_SERIAL_ERROR_MENU_STRUCTURE     "Error in menu structure"

#define MSG_ERR_EEPROM_WRITE                "Error writing to EEPROM!"

// temperature.cpp strings
#define MSG_PID_AUTOTUNE                    "PID Autotune"
#define MSG_PID_AUTOTUNE_START              MSG_PID_AUTOTUNE " start"
#define MSG_PID_AUTOTUNE_FAILED             MSG_PID_AUTOTUNE " failed!"
#define MSG_PID_BAD_EXTRUDER_NUM            MSG_PID_AUTOTUNE_FAILED " Bad extruder number"
#define MSG_PID_TEMP_TOO_HIGH               MSG_PID_AUTOTUNE_FAILED " Temperature too high"
#define MSG_PID_TIMEOUT                     MSG_PID_AUTOTUNE_FAILED " timeout"
#define MSG_BIAS                            " bias: "
#define MSG_D                               " d: "
#define MSG_T_MIN                           " min: "
#define MSG_T_MAX                           " max: "
#define MSG_KU                              " Ku: "
#define MSG_TU                              " Tu: "
#define MSG_CLASSIC_PID                     " Classic PID "
#define MSG_KP                              " Kp: "
#define MSG_KI                              " Ki: "
#define MSG_KD                              " Kd: "
#define MSG_KC                              " Kc: "
#define MSG_KF                              " Kf: "
#define MSG_B                               "B:"
#define MSG_T                               "T:"
#define MSG_AT                              " @:"
#define MSG_PID_AUTOTUNE_FINISHED           MSG_PID_AUTOTUNE " finished! Put the last Kp, Ki and Kd constants from below into Configuration.h"
#define MSG_PID_AUTOTUNE_UPDATED           	MSG_PID_AUTOTUNE " finished! Last Kp, Ki and Kd constants stored."
#define MSG_PID_DEBUG                       " PID_DEBUG "
#define MSG_PID_DEBUG_INPUT                 ": Input "
#define MSG_PID_DEBUG_OUTPUT                " Output "
#define MSG_PID_DEBUG_PTERM                 " pTerm "
#define MSG_PID_DEBUG_ITERM                 " iTerm "
#define MSG_PID_DEBUG_DTERM                 " dTerm "
#define MSG_PID_DEBUG_CTERM                 " cTerm "
#define MSG_PID_DEBUG_FTERM                 " fTerm "
#define MSG_INVALID_EXTRUDER_NUM            " - Invalid extruder number !"

#define MSG_HEATER_BED                      "bed"
#define MSG_STOPPED_HEATER                  ", system stopped! Heater_ID: "
#define MSG_REDUNDANCY                      "Heater switched off. Temperature difference between temp sensors is too high !"
#define MSG_T_HEATING_FAILED                "Heating failed"
#define MSG_T_THERMAL_RUNAWAY               "Thermal Runaway"
#define MSG_T_MAXTEMP                       "MAXTEMP triggered"
#define MSG_T_MINTEMP                       "MINTEMP triggered"

// Debug
#define MSG_DEBUG_PREFIX                    "DEBUG:"
#define MSG_DEBUG_OFF                       "off"
#define MSG_DEBUG_ECHO                      "ECHO"
#define MSG_DEBUG_INFO                      "INFO"
#define MSG_DEBUG_ERRORS                    "ERRORS"
#define MSG_DEBUG_DRYRUN                    "DRYRUN"
#define MSG_DEBUG_COMMUNICATION             "COMMUNICATION"
#define MSG_DEBUG_LEVELING                  "LEVELING"

// LCD Menu Messages

#if (!defined DISPLAY_CHARSET_HD44780_JAPAN && !defined DISPLAY_CHARSET_HD44780_WESTERN && !defined DISPLAY_CHARSET_HD44780_CYRILLIC )
  #define DISPLAY_CHARSET_HD44780_JAPAN
#endif

#include LANGUAGE_INCLUDE
#include "languages/language_en.h"

#endif //__LANGUAGE_H
