/*
    K8400 firmware modification based on original Version provided by Vellemann.

 Copyright (C) 2017 Holger Höhnemann

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include "planner.h"
#include "printer.h"
#include "temperature.h"
#include "ultralcd.h"
#include "fastio.h"

printer::printer()
{
	// TODO Auto-generated constructor stub
	 Status = Welcome;
	 blinker = false;
}

printer::~printer()
{
	// TODO Auto-generated destructor stub
}

void printer::message(const char* str)
{
	  char ch;
	  ch=pgm_read_byte(str);
	  while(ch)
	  {
	    MYSERIAL.write(ch);
	    ch=pgm_read_byte(++str);
	  }
}

void printer::message(char* str)
{
	MYSERIAL.write(str);
}

void printer::f_message(const char* formatstr, ...)
{
	  char ch;
	  int bufp = 0;
	  int formp = 0;
	  bool resolution = false;
	  int digits = 2;
	  int length=1;
	va_list ap;
	va_start (ap, formatstr);
	  ch=pgm_read_byte(formatstr);
	  while(ch)
	  {
		  if (ch == '%')
		  {
			  // handle format string
			  digits = 2;
			  formp=0;
			  format[formp++] = ch;
			  while (true)
			  {
				    ch=pgm_read_byte(++formatstr);
				    if (ch == '%')
				    {
						  buffer[bufp++] = ch;
						  break;
				    }
				    if (isdigit(ch))
				    {
				    	format[formp++] = ch;
				    	if (resolution)
				    	{
				    		digits = ch - '0';
				    	}
				    }
				    if (ch == '.')
				    {
				    	format[formp++] = ch;
				    	resolution = true;
				    }
				    if (ch == 'i' || ch == 'I')
				    {
				    	format[formp++] = ch;
				    	format[formp++] = '\0';
				    	bufp += sprintf(&buffer[bufp], &format[0], va_arg(ap, int));
//					    ch=pgm_read_byte(++formatstr);
				    	break;
				    }
				    if (ch == 'f' || ch == 'F')
				    {
				    	  size_t n = 0;
				    	  double number = va_arg(ap,double);

//				    	  if (isnan(number)) return print("nan");
//				    	  if (isinf(number)) return print("inf");
//				    	  if (number > 4294967040.0) return print ("ovf");  // constant determined empirically
//				    	  if (number <-4294967040.0) return print ("ovf");  // constant determined empirically

				    	  // Handle negative numbers
				    	  if (number < 0.0)
				    	  {
							  buffer[bufp++] = '-';
				    	     number = -number;
				    	  }

				    	  // Round correctly so that print(1.999, 2) prints as "2.00"
				    	  double rounding = 0.5;
				    	  for (uint8_t i=0; i<digits; ++i)
				    	    rounding /= 10.0;

				    	  number += rounding;

				    	  // Extract the integer part of the number and print it
				    	  unsigned long int_part = (unsigned long)number;
				    	  double remainder = number - (double)int_part;
					    	bufp += sprintf_P(&buffer[bufp], PSTR("%i"), int_part);
				    	  // Print the decimal point, but only if there are digits beyond
				    	  if (digits > 0) {
							  buffer[bufp++] = '.';
				    	  }

				    	  // Extract digits from the remainder one at a time
				    	  while (digits-- > 0)
				    	  {
				    	    remainder *= 10.0;
				    	    int toPrint = int(remainder);
				  		  buffer[bufp++] = '0'+toPrint;
				    	    remainder -= toPrint;
				    	  }

//						    ch=pgm_read_byte(++formatstr);
						    break;
				    }
				    if (ch == 's' || ch == 'S')
				    {
				    	char *str = va_arg(ap,char*);
				    	while (*str)
				    	{
				    	buffer[bufp++] = *str++;
				    	}
				    	break;
				    }
			  }
		  }
		  else
		  buffer[bufp++] = ch;
	    ch=pgm_read_byte(++formatstr);
	  }
	va_end (ap);
	  buffer[bufp++] = '\0';
	MYSERIAL.print(buffer);
}

void printer::Init()
{
	 MYSERIAL.begin(BAUDRATE);
	 Status = Welcome;
}

void printer::message_ln(const char* str)
{
	message(str);
    MYSERIAL.write('\n');
}

void printer::message_ln(char* str)
{
	message(str);
    MYSERIAL.write('\n');
}

void printer::Go_Idle()
{
	heater.Manage();

	if (max_inactive_time && Activity_Timeout())
		kill();

	if(Motor_Timeout())  {
		if( !plan.Blocks_Queued() )
		{
			disable_X();
			disable_Y();
			disable_Z();
			disable_E0();
			disable_E1();
		}

	}

#ifdef CHDK //Check if pin should be set to LOW after M240 set it to HIGH
	if (chdkActive)
	{
		chdkActive = false;
		if (millis()-chdkHigh < CHDK_DELAY) return;
		WRITE(CHDK, LOW);
	}
#endif

#if defined(KILL_PIN) && KILL_PIN > -1
	if( 0 == READ(KILL_PIN) )
		kill();
#endif
#if defined(CONTROLLERFAN_PIN) && CONTROLLERFAN_PIN > -1
	//	    controllerFan(); //Check if fan should be turned on to cool stepper drivers down
	if(!READ(X_ENABLE_PIN) || !READ(Y_ENABLE_PIN) || !READ(Z_ENABLE_PIN)
#if EXTRUDERS > 2
			|| !READ(E2_ENABLE_PIN)
#endif
#if EXTRUDERS > 1
#if defined(X2_ENABLE_PIN) && X2_ENABLE_PIN > -1
			|| !READ(X2_ENABLE_PIN)
#endif
			|| !READ(E1_ENABLE_PIN)
#endif
			|| !READ(E0_ENABLE_PIN)) //If any of the drivers are enabled...
	{
		Reset_Fan_Timeout();
	}
	if (Fan_Timeout())
	{
		digitalWrite(CONTROLLERFAN_PIN, 0);
		Write_Analogue(CONTROLLERFAN_PIN, 0);
	}
	else
	{
		// allows digital or PWM fan output to be used (see M42 handling)
		digitalWrite(CONTROLLERFAN_PIN, CONTROLLERFAN_SPEED);
		Write_Analogue(CONTROLLERFAN_PIN, CONTROLLERFAN_SPEED);
	}
#endif
#ifdef EXTRUDER_RUNOUT_PREVENT
	if( (millis() - previous_millis_cmd) >  EXTRUDER_RUNOUT_SECONDS*1000 )
		if(degHotend(active_extruder)>EXTRUDER_RUNOUT_MINTEMP)
		{
			bool oldstatus=READ(E0_ENABLE_PIN);
			enable_e0();
			float oldepos=current_position[E_AXIS];
			float oldedes=destination[E_AXIS];
			plan_buffer_line(destination[X_AXIS], destination[Y_AXIS], destination[Z_AXIS],
					destination[E_AXIS]+EXTRUDER_RUNOUT_EXTRUDE*EXTRUDER_RUNOUT_ESTEPS/axis_steps_per_unit[E_AXIS],
					EXTRUDER_RUNOUT_SPEED/60.*EXTRUDER_RUNOUT_ESTEPS/axis_steps_per_unit[E_AXIS], active_extruder);
			current_position[E_AXIS]=oldepos;
			destination[E_AXIS]=oldedes;
			plan_set_e_position(oldepos);
			previous_millis_cmd=millis();
			st_synchronize();
			WRITE(E0_ENABLE_PIN,oldstatus);
		}
#endif
#if defined(DUAL_X_CARRIAGE)
	// handle delayed move timeout
	if (delayed_move_time != 0 && (millis() - delayed_move_time) > 1000 && Stopped == false)
	{
		// travel moves have been received so enact them
		delayed_move_time = 0xFFFFFFFFUL; // force moves to be done
		memcpy(destination,current_position,sizeof(destination));
		prepare_move();
	}
#endif
#ifdef TEMP_STAT_LEDS
	handle_status_leds();
#endif
	plan.Check_Axes_Activity();
	lcd_update();
}

void printer::Reset_Activity_Timeout(unsigned long delay)
{
	max_inactive_time = delay;
	activity_timeout = millis() + delay;
}

void printer::Reset_Status_Timeout(unsigned long delay)
{
	status_timeout = millis() + delay;
}

void printer::Reset_Fan_Timeout(unsigned long delay)
{
	fan_timeout = millis() +delay;
}

void printer::Reset_MotorTimeout(unsigned long delay)
{
	stepper_inactive_time = delay;
	stepper_timeout = millis() + stepper_inactive_time;
}

void printer::Reset_LCD_Timeout(unsigned long delay)
{
	lcd_timeout = millis() + delay;
}
