/*
    K8400 firmware modification based on original Version provided by Vellemann.

 Copyright (C) 2017 Holger Höhnemann

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

//#include <planner_old.h>
#include "stepper.h"
#include "temperature.h"
#include "language.h"
#include "cardreader.h"
#include "MarlinHOH.h"
#include "speed_lookuptable.h"
#if defined(DIGIPOTSS_PIN) && DIGIPOTSS_PIN > -1
#include <SPI.h>
#endif
#include "parameter.h"
#include "printer.h"
#include "ultralcd.h"
//===========================================================================
//=============================   Interrupts      ============================
//===========================================================================
ISR (TIMER3_COMPA_vect)
{
	motor.isr_TIMER3_COMPA_vect();
}

volatile long endstops_trigsteps[3]={0,0,0};
volatile long endstops_stepsTotal,endstops_stepsDone;

#ifdef ABORT_ON_ENDSTOP_HIT_FEATURE_ENABLED
bool abort_on_endstop_hit = false;
#endif
#ifdef MOTOR_CURRENT_PWM_XY_PIN
  int motor_current_setting[3] = DEFAULT_PWM_MOTOR_CURRENT;
#endif

#if defined(X_MIN_PIN) && X_MIN_PIN > -1
static bool old_x_min_endstop=false;
#endif
static bool old_x_max_endstop=false;
#if defined(Y_MIN_PIN) && Y_MIN_PIN > -1
static bool old_y_min_endstop=false;
#endif

static bool old_y_max_endstop=false;
static bool old_z_min_endstop=false;

#if defined(Z_MAX_PIN) && Z_MAX_PIN > -1
static bool old_z_max_endstop=false;
#endif


volatile long count_position[NUM_AXIS] = { 0, 0, 0, 0};

#define CHECK_ENDSTOPS  if(motor.check_endstops)

// intRes = intIn1 * intIn2 >> 16
// uses:
// r26 to store 0
// r27 to store the byte 1 of the 24 bit result
#define MultiU16X8toH16(intRes, charIn1, intIn2) \
asm volatile ( \
"clr r26 \n\t" \
"mul %A1, %B2 \n\t" \
"movw %A0, r0 \n\t" \
"mul %A1, %A2 \n\t" \
"add %A0, r1 \n\t" \
"adc %B0, r26 \n\t" \
"lsr r0 \n\t" \
"adc %A0, r26 \n\t" \
"adc %B0, r26 \n\t" \
"clr r1 \n\t" \
: \
"=&r" (intRes) \
: \
"d" (charIn1), \
"d" (intIn2) \
: \
"r26" \
)

// intRes = longIn1 * longIn2 >> 24
// uses:
// r26 to store 0
// r27 to store the byte 1 of the 48bit result
#define MultiU24X24toH16(intRes, longIn1, longIn2) \
asm volatile ( \
"clr r26 \n\t" \
"mul %A1, %B2 \n\t" \
"mov r27, r1 \n\t" \
"mul %B1, %C2 \n\t" \
"movw %A0, r0 \n\t" \
"mul %C1, %C2 \n\t" \
"add %B0, r0 \n\t" \
"mul %C1, %B2 \n\t" \
"add %A0, r0 \n\t" \
"adc %B0, r1 \n\t" \
"mul %A1, %C2 \n\t" \
"add r27, r0 \n\t" \
"adc %A0, r1 \n\t" \
"adc %B0, r26 \n\t" \
"mul %B1, %B2 \n\t" \
"add r27, r0 \n\t" \
"adc %A0, r1 \n\t" \
"adc %B0, r26 \n\t" \
"mul %C1, %A2 \n\t" \
"add r27, r0 \n\t" \
"adc %A0, r1 \n\t" \
"adc %B0, r26 \n\t" \
"mul %B1, %A2 \n\t" \
"add r27, r1 \n\t" \
"adc %A0, r26 \n\t" \
"adc %B0, r26 \n\t" \
"lsr r27 \n\t" \
"adc %A0, r26 \n\t" \
"adc %B0, r26 \n\t" \
"clr r1 \n\t" \
: \
"=&r" (intRes) \
: \
"d" (longIn1), \
"d" (longIn2) \
: \
"r26" , "r27" \
)

// Some useful constants

#define ENABLE_STEPPER_DRIVER_INTERRUPT()  TIMSK3 |= (1<<OCIE3A)
#define DISABLE_STEPPER_DRIVER_INTERRUPT() TIMSK3 &= ~(1<<OCIE3A)


//         __________________________
//        /|                        |\     _________________         ^
//       / |                        | \   /|               |\        |
//      /  |                        |  \ / |               | \       s
//     /   |                        |   |  |               |  \      p
//    /    |                        |   |  |               |   \     e
//   +-----+------------------------+---+--+---------------+----+    e
//   |               BLOCK 1            |      BLOCK 2          |    d
//
//                           time ----->
//
//  The trapezoid is the shape the speed curve over time. It starts at block->initial_rate, accelerates
//  first block->accelerate_until step_events_completed, then keeps going at constant speed until
//  step_events_completed reaches block->decelerate_after after which it decelerates until the trapezoid generator is reset.
//  The slope of acceleration is calculated with the leib ramp alghorithm.

#ifdef ENABLE_AUTO_BED_LEVELING
float st_get_position_mm(uint8_t axis)
{
  float steper_position_in_steps = st_get_position(axis);
  return steper_position_in_steps / axis_steps_per_unit[axis];
}
#endif  // ENABLE_AUTO_BED_LEVELING


void digitalPotWrite(int address, int value) // From Arduino DigitalPotControl example
{
  #if defined(DIGIPOTSS_PIN) && DIGIPOTSS_PIN > -1
    digitalWrite(DIGIPOTSS_PIN,LOW); // take the SS pin low to select the chip
    SPI.transfer(address); //  send in the address and value via SPI:
    SPI.transfer(value);
    digitalWrite(DIGIPOTSS_PIN,HIGH); // take the SS pin high to de-select the chip:
    //delay(10);
  #endif
}

void digipot_init() //Initialize Digipot Motor Current
{
  #if defined(DIGIPOTSS_PIN) && DIGIPOTSS_PIN > -1
    const uint8_t digipot_motor_current[] = DIGIPOT_MOTOR_CURRENT;

    SPI.begin();
    pinMode(DIGIPOTSS_PIN, OUTPUT);
    for(int i=0;i<=4;i++)
      //digitalPotWrite(digipot_ch[i], digipot_motor_current[i]);
      digipot_current(i,digipot_motor_current[i]);
  #endif
  #ifdef MOTOR_CURRENT_PWM_XY_PIN
    pinMode(MOTOR_CURRENT_PWM_XY_PIN, OUTPUT);
    pinMode(MOTOR_CURRENT_PWM_Z_PIN, OUTPUT);
    pinMode(MOTOR_CURRENT_PWM_E_PIN, OUTPUT);
    digipot_current(0, motor_current_setting[0]);
    digipot_current(1, motor_current_setting[1]);
    digipot_current(2, motor_current_setting[2]);
    //Set timer5 to 31khz so the PWM of the motor power is as constant as possible. (removes a buzzing noise)
    TCCR5B = (TCCR5B & ~(_BV(CS50) | _BV(CS51) | _BV(CS52))) | _BV(CS50);
  #endif
}

void digipot_current(uint8_t driver, int current)
{
  #if defined(DIGIPOTSS_PIN) && DIGIPOTSS_PIN > -1
    const uint8_t digipot_ch[] = DIGIPOT_CHANNELS;
    digitalPotWrite(digipot_ch[driver], current);
  #endif
  #ifdef MOTOR_CURRENT_PWM_XY_PIN
  if (driver == 0) analogWrite(MOTOR_CURRENT_PWM_XY_PIN, (long)current * 255L / (long)MOTOR_CURRENT_PWM_RANGE);
  if (driver == 1) analogWrite(MOTOR_CURRENT_PWM_Z_PIN, (long)current * 255L / (long)MOTOR_CURRENT_PWM_RANGE);
  if (driver == 2) analogWrite(MOTOR_CURRENT_PWM_E_PIN, (long)current * 255L / (long)MOTOR_CURRENT_PWM_RANGE);
  #endif
}

void microstep_init()
{
#if defined(X_MS1_PIN) && X_MS1_PIN > -1
  const uint8_t microstep_modes[] = MICROSTEP_MODES;
#endif

  #if defined(E1_MS1_PIN) && E1_MS1_PIN > -1
  pinMode(E1_MS1_PIN,OUTPUT);
  pinMode(E1_MS2_PIN,OUTPUT); 
  #endif

  #if defined(X_MS1_PIN) && X_MS1_PIN > -1
  pinMode(X_MS1_PIN,OUTPUT);
  pinMode(X_MS2_PIN,OUTPUT);  
  pinMode(Y_MS1_PIN,OUTPUT);
  pinMode(Y_MS2_PIN,OUTPUT);
  pinMode(Z_MS1_PIN,OUTPUT);
  pinMode(Z_MS2_PIN,OUTPUT);  
  pinMode(E0_MS1_PIN,OUTPUT);
  pinMode(E0_MS2_PIN,OUTPUT);
  for(int i=0;i<=4;i++) microstep_mode(i,microstep_modes[i]);
  #endif
}

void microstep_ms(uint8_t driver, int8_t ms1, int8_t ms2)
{
  if(ms1 > -1) switch(driver)
  {
    case 0: digitalWrite( X_MS1_PIN,ms1); break;
    case 1: digitalWrite( Y_MS1_PIN,ms1); break;
    case 2: digitalWrite( Z_MS1_PIN,ms1); break;
    case 3: digitalWrite(E0_MS1_PIN,ms1); break;
    #if defined(E1_MS1_PIN) && E1_MS1_PIN > -1
    case 4: digitalWrite(E1_MS1_PIN,ms1); break;
    #endif
  }
  if(ms2 > -1) switch(driver)
  {
    case 0: digitalWrite( X_MS2_PIN,ms2); break;
    case 1: digitalWrite( Y_MS2_PIN,ms2); break;
    case 2: digitalWrite( Z_MS2_PIN,ms2); break;
    case 3: digitalWrite(E0_MS2_PIN,ms2); break;
    #if defined(E1_MS2_PIN) && E1_MS2_PIN > -1
    case 4: digitalWrite(E1_MS2_PIN,ms2); break;
    #endif
  }
}

void microstep_mode(uint8_t driver, uint8_t stepping_mode)
{
  switch(stepping_mode)
  {
    case 1: microstep_ms(driver,MICROSTEP1); break;
    case 2: microstep_ms(driver,MICROSTEP2); break;
    case 4: microstep_ms(driver,MICROSTEP4); break;
    case 8: microstep_ms(driver,MICROSTEP8); break;
    case 16: microstep_ms(driver,MICROSTEP16); break;
  }
}

void microstep_readings()
{
      SERIAL_PROTOCOLPGM("MS1,MS2 Pins\n");
      SERIAL_PROTOCOLPGM("X: ");
      SERIAL_PROTOCOL(   digitalRead(X_MS1_PIN));
      SERIAL_PROTOCOLLN( digitalRead(X_MS2_PIN));
      SERIAL_PROTOCOLPGM("Y: ");
      SERIAL_PROTOCOL(   digitalRead(Y_MS1_PIN));
      SERIAL_PROTOCOLLN( digitalRead(Y_MS2_PIN));
      SERIAL_PROTOCOLPGM("Z: ");
      SERIAL_PROTOCOL(   digitalRead(Z_MS1_PIN));
      SERIAL_PROTOCOLLN( digitalRead(Z_MS2_PIN));
      SERIAL_PROTOCOLPGM("E0: ");
      SERIAL_PROTOCOL(   digitalRead(E0_MS1_PIN));
      SERIAL_PROTOCOLLN( digitalRead(E0_MS2_PIN));
      #if defined(E1_MS1_PIN) && E1_MS1_PIN > -1
      SERIAL_PROTOCOLPGM("E1: ");
      SERIAL_PROTOCOL(   digitalRead(E1_MS1_PIN));
      SERIAL_PROTOCOLLN( digitalRead(E1_MS2_PIN));
      #endif
}

stepper::stepper()
{
	check_endstops = true;
	OCR3A_nominal = 100;
	acc_step_rate =0;
	acceleration_time = deceleration_time = 0;
	out_bits = 0;
	step_events_completed = 0;
	step_loops = 0;
	step_loops_nominal = 0;
}

stepper::~stepper()
{
}

void checkStepperErrors()
{
}

void stepper::digipot_init()
{
}

void stepper::Enable_Endstops(bool check)
{
	  check_endstops = check;
}

void stepper::Set_Position(const long & x, const long & y, const long & z,
		const long & e)
{
	  CRITICAL_SECTION_START;
	  count_position[X_AXIS] = x;
	  count_position[Y_AXIS] = y;
	  count_position[Z_AXIS] = z;
	  count_position[E_AXIS] = e;
	  CRITICAL_SECTION_END;
}

void stepper::Set_E_Position(const long & e)
{
	  CRITICAL_SECTION_START;
	  count_position[E_AXIS] = e;
	  CRITICAL_SECTION_END;
}

void stepper::Set_Position (const AxisEnum axis,  const long & v)
{
	  CRITICAL_SECTION_START;
	  count_position[axis] = v;
	  CRITICAL_SECTION_END;
}



void stepper::Wake_Up()
{
	  //  TCNT1 = 0;
	  ENABLE_STEPPER_DRIVER_INTERRUPT();
}

void stepper::FinishAndDisable()
{
	  Synchronize();
	  disable_X();
	  disable_Y();
	  disable_Z();
	  disable_E0();
	  disable_E1();
	  disable_E2();
}

long stepper::Get_Position(AxisEnum axis)
{
	  long count_pos;
	  CRITICAL_SECTION_START;
	  count_pos = count_position[axis];
	  CRITICAL_SECTION_END;
	  return count_pos;
}

void stepper::microstep_init()
{
}

void stepper::Initialize()
{
	 //Initialize Dir Pins
	  #if defined(X_DIR_PIN) && X_DIR_PIN > -1
	    SET_OUTPUT(X_DIR_PIN);
	  #endif
	  #if defined(Y_DIR_PIN) && Y_DIR_PIN > -1
	    SET_OUTPUT(Y_DIR_PIN);
	  #endif
	  #if defined(Z_DIR_PIN) && Z_DIR_PIN > -1
	    SET_OUTPUT(Z_DIR_PIN);
	  #endif
	  #if defined(E0_DIR_PIN) && E0_DIR_PIN > -1
	    SET_OUTPUT(E0_DIR_PIN);
	  #endif
	  #if defined(E1_DIR_PIN) && (E1_DIR_PIN > -1)
	    SET_OUTPUT(E1_DIR_PIN);
	  #endif

	  //Initialize Enable Pins - steppers default to disabled.

	  #if defined(X_ENABLE_PIN) && X_ENABLE_PIN > -1
	    SET_OUTPUT(X_ENABLE_PIN);
	    if(!X_ENABLE_ON) WRITE(X_ENABLE_PIN,HIGH);
	  #endif
	  #if defined(Y_ENABLE_PIN) && Y_ENABLE_PIN > -1
	    SET_OUTPUT(Y_ENABLE_PIN);
	    if(!Y_ENABLE_ON) WRITE(Y_ENABLE_PIN,HIGH);
	  #endif
	  #if defined(Z_ENABLE_PIN) && Z_ENABLE_PIN > -1
	    SET_OUTPUT(Z_ENABLE_PIN);
	    if(!Z_ENABLE_ON) WRITE(Z_ENABLE_PIN,HIGH);
	  #endif
	  #if defined(E0_ENABLE_PIN) && (E0_ENABLE_PIN > -1)
	    SET_OUTPUT(E0_ENABLE_PIN);
	    if(!E_ENABLE_ON) WRITE(E0_ENABLE_PIN,HIGH);
	  #endif
	  #if defined(E1_ENABLE_PIN) && (E1_ENABLE_PIN > -1)
	    SET_OUTPUT(E1_ENABLE_PIN);
	    if(!E_ENABLE_ON) WRITE(E1_ENABLE_PIN,HIGH);
	  #endif
	  //endstops and pullups

	  #if defined(Z_MIN_PIN) && Z_MIN_PIN > -1
	    SET_INPUT(Z_MIN_PIN);
	    #ifdef ENDSTOPPULLUP_ZMIN
	      WRITE(Z_MIN_PIN,HIGH);
	    #endif
	  #endif

	  #if defined(X_MAX_PIN) && X_MAX_PIN > -1
	    SET_INPUT(X_MAX_PIN);
	    #ifdef ENDSTOPPULLUP_XMAX
	      WRITE(X_MAX_PIN,HIGH);
	    #endif
	  #endif

	  #if defined(Y_MAX_PIN) && Y_MAX_PIN > -1
	    SET_INPUT(Y_MAX_PIN);
	    #ifdef ENDSTOPPULLUP_YMAX
	      WRITE(Y_MAX_PIN,HIGH);
	    #endif
	  #endif

	  #if defined(Z_MAX_PIN) && Z_MAX_PIN > -1
	    SET_INPUT(Z_MAX_PIN);
	    #ifdef ENDSTOPPULLUP_ZMAX
	      WRITE(Z_MAX_PIN,HIGH);
	    #endif
	  #endif


	  //Initialize Step Pins
	  #if defined(X_STEP_PIN) && (X_STEP_PIN > -1)
	    SET_OUTPUT(X_STEP_PIN);
	    WRITE(X_STEP_PIN,INVERT_X_STEP_PIN);
	    disable_X();
	  #endif
	  #if defined(X2_STEP_PIN) && (X2_STEP_PIN > -1)
	    SET_OUTPUT(X2_STEP_PIN);
	    WRITE(X2_STEP_PIN,INVERT_X_STEP_PIN);
	    disable_x();
	  #endif
	  #if defined(Y_STEP_PIN) && (Y_STEP_PIN > -1)
	    SET_OUTPUT(Y_STEP_PIN);
	    WRITE(Y_STEP_PIN,INVERT_Y_STEP_PIN);
	    #if defined(Y_DUAL_STEPPER_DRIVERS) && defined(Y2_STEP_PIN) && (Y2_STEP_PIN > -1)
	      SET_OUTPUT(Y2_STEP_PIN);
	      WRITE(Y2_STEP_PIN,INVERT_Y_STEP_PIN);
	    #endif
	    disable_Y();
	  #endif
	  #if defined(Z_STEP_PIN) && (Z_STEP_PIN > -1)
	    SET_OUTPUT(Z_STEP_PIN);
	    WRITE(Z_STEP_PIN,INVERT_Z_STEP_PIN);
	    #if defined(Z_DUAL_STEPPER_DRIVERS) && defined(Z2_STEP_PIN) && (Z2_STEP_PIN > -1)
	      SET_OUTPUT(Z2_STEP_PIN);
	      WRITE(Z2_STEP_PIN,INVERT_Z_STEP_PIN);
	    #endif
	    disable_Z();
	  #endif
	  #if defined(E0_STEP_PIN) && (E0_STEP_PIN > -1)
	    SET_OUTPUT(E0_STEP_PIN);
	    WRITE(E0_STEP_PIN,INVERT_E_STEP_PIN);
	    disable_E0();
	  #endif
	  #if defined(E1_STEP_PIN) && (E1_STEP_PIN > -1)
	    SET_OUTPUT(E1_STEP_PIN);
	    WRITE(E1_STEP_PIN,INVERT_E_STEP_PIN);
	    disable_E1();
	  #endif

	  // for timer 3
	  // waveform generation = 0100 = CTC
	  TCCR3B &= ~(1<<WGM33);
	  TCCR3B |=  (1<<WGM32);
	  TCCR3A &= ~(1<<WGM31);
	  TCCR3A &= ~(1<<WGM30);

	  // output mode = 00 (disconnected)
	  TCCR3A &= ~(3<<COM3A0);
	  TCCR3A &= ~(3<<COM3B0);

	  TCCR3B = (TCCR3B & ~(0x07<<CS10)) | (2<<CS10);

	  OCR3A = 0x4000;
	  TCNT3 = 0;
	  TIMSK3 |= (1<<OCIE3A) ; // enable stepper interrupt

	  Enable_Endstops(true); // Start with endstops active. After homing they can be disabled
	  sei();

}

void stepper::QuickStop()
{
	  DISABLE_STEPPER_DRIVER_INTERRUPT();
	  while(plan.Blocks_Queued())
	    plan.Discard_Current_Block();
	  current_block = NULL;
	  ENABLE_STEPPER_DRIVER_INTERRUPT();
}

void stepper::Synchronize()
{
	while( plan.Blocks_Queued())
		my_printer.Go_Idle();
}


inline void __attribute__((always_inline)) stepper::Reset_trapezoid_generator()
{
	deceleration_time = 0;
	// step_rate to timer interval
	OCR3A_nominal = calc_timer(current_block->nominal_rate);
	// make a note of the number of step loops required at nominal speed
	step_loops_nominal = step_loops;
	acc_step_rate = current_block->initial_rate;
	acceleration_time = calc_timer(acc_step_rate);
	OCR3A = acceleration_time;
}

inline void stepper::isr_TIMER3_COMPA_vect()
{
	// If there is no current block, attempt to pop one from the buffer
	if (current_block == NULL)
	{
		// Anything in the buffer?
		current_block = plan.Get_Current_Block();
		if (current_block)
		{
			// initialöize the block
			// Set directions TO DO This should be done once during init of trapezoid. Endstops -> interrupt
			out_bits = current_block->direction_bits;

			current_block->busy = true;
			Reset_trapezoid_generator();
			counter_x = -(current_block->step_event_count >> 1);
			counter_y = counter_x;
			counter_z = counter_x;
			counter_e = counter_x;
			step_events_completed = 0;

			// Set the direction bits (X_AXIS=A_AXIS and Y_AXIS=B_AXIS for COREXY)
			if((out_bits & (1<<X_AXIS))!=0)
			{
				WRITE(X_DIR_PIN, INVERT_X_DIR);
				count_direction[X_AXIS]=-1;
			}
			else
			{
				WRITE(X_DIR_PIN, !INVERT_X_DIR);
				count_direction[X_AXIS]=1;
			}
			if((out_bits & (1<<Y_AXIS))!=0)
			{
				WRITE(Y_DIR_PIN, INVERT_Y_DIR);
				count_direction[Y_AXIS]=-1;
			}
			else
			{
				WRITE(Y_DIR_PIN, !INVERT_Y_DIR);
				count_direction[Y_AXIS]=1;
			}
			if ((out_bits & (1<<Z_AXIS)) != 0)
			{   // -direction
				WRITE(Z_DIR_PIN,INVERT_Z_DIR);
				count_direction[Z_AXIS]=-1;
			}
			else
			{ // +direction
				WRITE(Z_DIR_PIN,!INVERT_Z_DIR);
				count_direction[Z_AXIS]=1;
			}
			if ((out_bits & (1<<E_AXIS)) != 0)
			{  // -direction
				REV_E_DIR();
				count_direction[E_AXIS]=-1;
			}
			else
			{ // +direction
				NORM_E_DIR();
				count_direction[E_AXIS]=1;
			}
		}

	}
	if (current_block == NULL)
	{
		// still nothing, wait ....
		OCR3A=2000; // 1kHz., wait 1 ms and check again for a new block
	}
	else
	{
		// Check limit switches
		if ((out_bits & (1<<X_AXIS)) != 0) {   // stepping along -X axis
			CHECK_ENDSTOPS
			{
			}
		}
		else { // +direction
			CHECK_ENDSTOPS
			{
				{
#if defined(X_MAX_PIN) && X_MAX_PIN > -1
					bool x_max_endstop=(READ(X_MAX_PIN) != X_MAX_ENDSTOP_INVERTING);
					if(x_max_endstop && old_x_max_endstop && (current_block->steps[X_AXIS] > 0)){
						endstops_trigsteps[X_AXIS] = count_position[X_AXIS];
						endstop_x_hit=true;
						step_events_completed = current_block->step_event_count;
					}
					old_x_max_endstop = x_max_endstop;
#endif
				}
			}
		}

		if ((out_bits & (1<<Y_AXIS)) != 0) {   // -direction
			CHECK_ENDSTOPS
			{
			}
		}
		else { // +direction
			CHECK_ENDSTOPS
			{
#if defined(Y_MAX_PIN) && Y_MAX_PIN > -1
				bool y_max_endstop=(READ(Y_MAX_PIN) != Y_MAX_ENDSTOP_INVERTING);
				if(y_max_endstop && old_y_max_endstop && (current_block->steps[Y_AXIS] > 0)){
					endstops_trigsteps[Y_AXIS] = count_position[Y_AXIS];
					endstop_y_hit=true;
					step_events_completed = current_block->step_event_count;
				}
				old_y_max_endstop = y_max_endstop;
#endif
			}
		}

		if ((out_bits & (1<<Z_AXIS)) != 0) {   // -direction
			CHECK_ENDSTOPS
			{
#if defined(Z_MIN_PIN) && Z_MIN_PIN > -1
				bool z_min_endstop=(READ(Z_MIN_PIN) != Z_MIN_ENDSTOP_INVERTING);
				if(z_min_endstop && old_z_min_endstop && (current_block->steps[Z_AXIS] > 0)) {
					endstops_trigsteps[Z_AXIS] = count_position[Z_AXIS];
					endstop_z_hit=true;
					step_events_completed = current_block->step_event_count;
				}
				old_z_min_endstop = z_min_endstop;
#endif
			}
		}
		else { // +direction
			CHECK_ENDSTOPS
			{
			}
		}

#ifdef PRESSURE_ADVANCE
		// initial shift as fast as possible and as one block
		if ( current_block->initial_advance > 0)
		{
			if (count_direction[E_AXIS] == 1)	// only in forward direction
			{
				//			NORM_E_DIR();     					// restore original direction
				while (current_block->initial_advance > 0)
				{
					WRITE_E_STEP(!INVERT_E_STEP_PIN);
					current_block->initial_advance -= 1;
					WRITE_E_STEP(INVERT_E_STEP_PIN);
					// delay to avoid slipping
					delayMicroseconds(config.advance_delay[current_block->active_extruder]);
				}
			}
		}
		else
#endif // PRESSURE_ADVANCE
		{
			for(int8_t i=0; i < step_loops; i++)
			{ // Take multiple steps per interrupt (For high speed moves)
#ifndef AT90USB
				MSerial.checkRx(); // Check for serial chars.
#endif


				counter_x += current_block->steps[X_AXIS];
				if (counter_x > 0) {
					WRITE(X_STEP_PIN, !INVERT_X_STEP_PIN);
					counter_x -= current_block->step_event_count;
					count_position[X_AXIS]+=count_direction[X_AXIS];
					WRITE(X_STEP_PIN, INVERT_X_STEP_PIN);
				}

				counter_y += current_block->steps[Y_AXIS];
				if (counter_y > 0) {
					WRITE(Y_STEP_PIN, !INVERT_Y_STEP_PIN);
					counter_y -= current_block->step_event_count;
					count_position[Y_AXIS]+=count_direction[Y_AXIS];
					WRITE(Y_STEP_PIN, INVERT_Y_STEP_PIN);
				}

				counter_z += current_block->steps[Z_AXIS];
				if (counter_z > 0) {
					WRITE(Z_STEP_PIN, !INVERT_Z_STEP_PIN);
					counter_z -= current_block->step_event_count;
					count_position[Z_AXIS]+=count_direction[Z_AXIS];
					WRITE(Z_STEP_PIN, INVERT_Z_STEP_PIN);
				}

				counter_e += current_block->steps[E_AXIS];
				if (counter_e > 0) {
					WRITE_E_STEP(!INVERT_E_STEP_PIN);
					counter_e -= current_block->step_event_count;
					count_position[E_AXIS]+=count_direction[E_AXIS];
					WRITE_E_STEP(INVERT_E_STEP_PIN);
				}

				step_events_completed += 1;
				if(step_events_completed >= current_block->step_event_count) break;
			}
			// Calculate new timer value
			unsigned short timer;
			unsigned short step_rate;
			if (step_events_completed <= (unsigned long int)current_block->accelerate_until) {

				MultiU24X24toH16(acc_step_rate, acceleration_time, current_block->acceleration_rate);
				acc_step_rate += current_block->initial_rate;

				// upper limit
				if(acc_step_rate > current_block->nominal_rate)
					acc_step_rate = current_block->nominal_rate;

				// step_rate to timer interval
				timer = calc_timer(acc_step_rate);
				OCR3A = timer;
				acceleration_time += timer;
			}
			else if (step_events_completed > (unsigned long int)current_block->decelerate_after) {
				MultiU24X24toH16(step_rate, deceleration_time, current_block->acceleration_rate);

				if(step_rate > acc_step_rate) { // Check step_rate stays positive
					step_rate = current_block->final_rate;
				}
				else {
					step_rate = acc_step_rate - step_rate; // Decelerate from aceleration end point.
				}

				// lower limit
				if(step_rate < current_block->final_rate)
					step_rate = current_block->final_rate;

				// step_rate to timer interval
				timer = calc_timer(step_rate);
				OCR3A = timer;
				deceleration_time += timer;
			}
			else {
				OCR3A = OCR3A_nominal;
				// ensure we're running at the correct step rate, even if we just came off an acceleration
				step_loops = step_loops_nominal;
			}



			// If current block is finished, reset pointer
			if (step_events_completed >= current_block->step_event_count) {
#ifdef PRESSURE_ADVANCE
				// release pressure after finalisation of one block in one turn
				if(current_block->final_advance < 0 )
				{
					if (count_direction[E_AXIS]==1) 	// double check if we are still in extrusion condition
					{
						REV_E_DIR();						// set opposite extrusion direction
						while (current_block->final_advance <0)
						{
							WRITE_E_STEP(!INVERT_E_STEP_PIN);	// run release step
							current_block->final_advance += 1;	// increment counter (final_advance < 0!)
							WRITE_E_STEP(INVERT_E_STEP_PIN);
							// delay to avoid slipping
							delayMicroseconds(config.advance_delay[current_block->active_extruder]);
						}
						NORM_E_DIR();     					// restore original direction
					}
				}
#endif // PRESSURE_ADVANCE

				current_block = NULL;
				plan.Discard_Current_Block();
			}
		}
	}
}

void stepper::Endstops_hit_on_Purpose()
{
	  endstop_x_hit=false;
	  endstop_y_hit=false;
	  endstop_z_hit=false;
}

void stepper::CheckHitEndstops()
{

	 if( endstop_x_hit || endstop_y_hit || endstop_z_hit) {
		 my_printer.echo(T(MSG_Endstops_Hit));
	   if(endstop_x_hit) {
		   my_printer.message(PSTR(" X \n"));
	   }
	   if(endstop_y_hit) {
		   my_printer.message(PSTR(" Y \n"));
	   }
	   if(endstop_z_hit) {
		   my_printer.message(PSTR(" Z \n"));
	   }
	   endstop_x_hit=false;
	   endstop_y_hit=false;
	   endstop_z_hit=false;
	#if defined(ABORT_ON_ENDSTOP_HIT_FEATURE_ENABLED) && defined(SDSUPPORT)
	   if (abort_on_endstop_hit)
	   {
	     card.sdprinting = false;
	     card.closefile();
	     quickStop();
	     setTargetHotend0(0);
	     setTargetHotend1(0);
	     setTargetHotend2(0);
	   }
	#endif
	 }

}

inline unsigned short stepper::calc_timer(unsigned short step_rate)
{
	  unsigned short timer;
	  if(step_rate > MAX_STEP_FREQUENCY) step_rate = MAX_STEP_FREQUENCY;

	  if(step_rate > 20000) { // If steprate > 20kHz >> step 4 times
	    step_rate = (step_rate >> 2)&0x3fff;
	    step_loops = 4;
	  }
	  else if(step_rate > 10000) { // If steprate > 10kHz >> step 2 times
	    step_rate = (step_rate >> 1)&0x7fff;
	    step_loops = 2;
	  }
	  else {
	    step_loops = 1;
	  }

	  if(step_rate < (F_CPU/500000)) step_rate = (F_CPU/500000);
	  step_rate -= (F_CPU/500000); // Correct for minimal speed
	  if(step_rate >= (8*256)){ // higher step rate
	    unsigned short table_address = (unsigned short)&speed_lookuptable_fast[(unsigned char)(step_rate>>8)][0];
	    unsigned char tmp_step_rate = (step_rate & 0x00ff);
	    unsigned short gain = (unsigned short)pgm_read_word_near(table_address+2);
	    MultiU16X8toH16(timer, tmp_step_rate, gain);
	    timer = (unsigned short)pgm_read_word_near(table_address) - timer;
	  }
	  else { // lower step rates
	    unsigned short table_address = (unsigned short)&speed_lookuptable_slow[0][0];
	    table_address += ((step_rate)>>1) & 0xfffc;
	    timer = (unsigned short)pgm_read_word_near(table_address);
	    timer -= (((unsigned short)pgm_read_word_near(table_address+2) * (unsigned char)(step_rate & 0x0007))>>3);
	  }
	  if(timer < 100) { timer = 100; MYSERIAL.print(MSG_STEPPER_TOO_HIGH); MYSERIAL.println(step_rate); }//(20kHz this should never happen)
	  return timer;

}
